<?php
/**
 * Application level Controller
 *
 * This file is application-wide controller file. You can put all
 * application-wide controller-related methods here.
 *
 * CakePHP(tm) : Rapid Development Framework (http://cakephp.org)
 * Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright     Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 * @link          http://cakephp.org CakePHP(tm) Project
 * @package       app.Controller
 * @since         CakePHP(tm) v 0.2.9
 * @license       http://www.opensource.org/licenses/mit-license.php MIT License
 */
App::uses('Controller', 'Controller');

/**
 * Application Controller
 *
 * Add your application-wide methods in the class below, your controllers
 * will inherit them.
 *
 * @package         app.Controller
 * @link        http://book.cakephp.org/2.0/en/controllers.html#the-app-controller
 */
class AppController extends Controller
{

    public $systemSettings = array();
    public $components = array(
        'Cookie',
        'Session',
        'DebugKit.Toolbar',
        'Auth' => array(
            'authenticate' => array(
                'Form' => array(
                    'fields' => array('username' => 'email'),
                    'passwordHasher' => 'Blowfish',
                ),
            ),
            'authorize' => 'Controller',
            'flash' => array(
                'element' => 'alert',
                'key' => 'auth',
                'params' => array(
                    'plugin' => 'BoostCake',
                    'class' => 'alert-warning',
                ),
            ),
        ),
    );
    public $helpers = array(
        'Session',
        'Html' => array('className' => 'BoostCake.BoostCakeHtml'),
        'Form' => array('className' => 'BoostCake.BoostCakeForm'),
        'Paginator' => array('className' => 'BoostCake.BoostCakePaginator'),
    );

    public function beforeFilter()
    {
        $this->Cookie->time = '30 Days'; // or '1 hour'
        $this->Cookie->key = 'AS()XA(S*D)AS8dA(Sd80A(SDA*SDAS%D4$AS#SD@ASDtyASGH)_AS0dAoIASNKAshgaFA$#S21d24a3s45dAS$3d#A@$SDASCHVASCa4s33%$ˆ$%$#s253$AS5#Â$%s645$#AS@%#AˆS6%A&*SÂ%S$';
        $this->Cookie->httpOnly = true;

        $this->initAuth();

        $this->initLayout();

        $this->initSettings();

        if ($this->params->params['controller'] == 'users' && $this->params->params['action'] == 'login' && AuthComponent::user('id')) {
            $this->redirect($this->Auth->redirectUrl());
        }
    }

    public function initLayout()
    {
        if ($this->request->prefix == 'admin') {
            $this->layout = 'admin';
        }
    }

    public function initAuth()
    {
        $this->Auth->loginAction = array('action' => 'login', 'controller' => 'users');
        $this->Auth->loginRedirect = array('controller' => 'system', 'action' => 'dashboard');
        $this->Auth->logoutRedirect = array('action' => 'login', 'controller' => 'users');
        $this->Auth->unauthorizedRedirect = array('controller' => 'pages', 'action' => 'unauthorized', 'admin' => false);
        $this->Auth->authError = 'Please login to continue.';
    }

    public function isAuthorized($user = null)
    {
        // Any registered user can access public functions
        if (empty($this->request->params['admin'])) {
            return true;
        }

        // Only admins can access admin functions
        if (isset($this->request->params['admin'])) {
            return (bool) ($user['is_admin']);
        }

        // Default deny
        return false;
    }

    public function initSettings()
    {
        $this->loadModel('SystemSetting');
        $system_settings = $this->SystemSetting->findById(1);
        $this->systemSettings = $system_settings['SystemSetting'];
        $this->set('system_settings', $system_settings['SystemSetting']);

        $this->loadModel('Pages');
        $right_section = $this->Pages->findBySlug('right_section');
        $this->set('right_section', $right_section['Pages']['content']);
    }

    public function afterPaypalNotification($txnId)
    {
        //Here is where you can implement code to apply the transaction to your app.
        //for example, you could now mark an order as paid, a subscription, or give the user premium access.
        //retrieve the transaction using the txnId passed and apply whatever logic your site needs.

        $transaction = ClassRegistry::init('PaypalIpn.InstantPaymentNotification')->findById($txnId);
        $this->log($transaction['InstantPaymentNotification']['id'], 'paypal');

        //Tip: be sure to check the payment_status is complete because failure
        //     are also saved to your database for review.

        if ($transaction['InstantPaymentNotification']['payment_status'] == 'Completed') {

            //Yay!  We have monies!
            $id = $transaction['InstantPaymentNotification']['custom'];

            // Update payment
            $this->loadModel('Payment');
            $this->Payment->id = $id;
            $this->Payment->set('status', 'Paid');
            $this->Payment->save();
            
            // Update user
            $this->loadModel('User');
            $this->User->id = $this->Payment->field('user_id');
            $this->User->set('is_subscribed', 1);
            $today = date('Y-m-d H:i:s');
            $days = $this->systemSettings['fee_days'];
            $new_expiry_date = date('Y-m-d H:i:s', strtotime($today . " + $days days"));
            $this->User->set('expiry_date', $new_expiry_date);
            $this->User->save();
        } else {
            //Oh no, better look at this transaction to determine what to do; like email a decline letter.
            $this->log('Payment not completed');
        }
    }
}
