<?php
App::uses('AppController', 'Controller');
App::uses('CakeEmail', 'Network/Email');

/**
 * Applicants Controller
 *
 * Use DataTables
 * 
 * @property Applicant $Applicant
 * @property PaginatorComponent $Paginator
 */
class ApplicantsController extends AppController
{

    /**
     * Components
     *
     * @var array
     */
    public $components = array('Paginator', 'Search.Prg', 'CsvView.CsvView', 'AjaxForms', 'RequestHandler' => array(
            'viewClassMap' => array('csv' => 'CsvView.Csv')
    ));

    public function beforeFilter()
    {
        parent::beforeFilter();

        $this->Auth->allow('add', 'add_ajax');
        // Only enable AjaxForms when needed
        $this->AjaxForms->enabled = false;

        if (isset($this->AjaxForms) && $this->RequestHandler->isAjax() && in_array($this->action, array('admin_add_ajax', 'admin_edit_ajax', 'add_ajax', 'edit_ajax'))) {
            $this->AjaxForms->enabled = true;
        }

        // Use ajax layout when Ajax used
        if ($this->RequestHandler->isAjax()) {
            $this->layout = 'ajax';
        }
    }

    /**
     * Helpers
     */
    public $helpers = array('CustomForm', 'Editor' => array(
            'className' => 'Summernote.Summernote'
    ));

    /**
     * Pagination Options
     */
    public $paginate = array(
        'limit' => 10,
        'order' => array(
            'Applicant.name' => 'asc'
        ),
        'paramType' => 'querystring',
    );

    public function admin_load_data()
    {
        ini_set('memory_limit', '-1');
        $this->Applicant->recursive = 0;

        $this->Prg->commonProcess();

        $applicants = $this->Applicant->find('all', array(
            'conditions' => $this->Applicant->parseCriteria($this->Prg->parsedParams()),
        ));

        $this->set('data', $applicants);
        $this->set('status', 'success');
        $this->set('message', 'Record(s) found');
        $this->set('_serialize', array('data', 'status', 'message'));
    }

    /**
     * index method
     *
     * @return void
     */
    public function admin_index_normaltable()
    {
        $this->Applicant->recursive = 0;

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->Applicant->parseCriteria($this->Prg->parsedParams());

        $this->set('applicants', $this->Paginator->paginate());
    }

    public function admin_index()
    {
        $this->Applicant->recursive = 0;

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->Applicant->parseCriteria($this->Prg->parsedParams());

        $this->set('applicants', $this->Paginator->paginate());
    }

    /**
     * admin_print method
     */
    public function admin_print()
    {
        Configure::write('debug', 0);
        $this->layout = 'print';
        $this->Applicant->recursive = 0;

        $this->pdfConfig = array(
            'orientation' => 'portrait',
            'filename' => 'Applicants_List',
        );

        $this->set('applicants', $this->Applicant->find('all', array('order' => 'Applicant.name ASC')));
    }

    /**
     * admin_export method
     */
    public function admin_export()
    {
//        $applicants = $this->Applicant->find('all');
//        $_serialize = 'applicants';
//        $_header = array('ID', 'Name', 'Applicantname', 'Email');
//        $_extract = array('Applicant.id', 'Applicant.name', 'Applicant.applicantname', 'Applicant.email');
//
//        $this->response->download('applicants.csv');
//        $this->set(compact('applicants', '_serialize', '_header', '_extract'));

        $results = $this->Applicant->find('all');

        $excludePaths = array('Applicant.password'); // Exclude all id fields
//        $customHeaders = array('City.population' => 'No. of People');

        $this->CsvView->quickExport($results, $excludePaths);
//        $this->CsvView->quickExport($results);
    }

    /**
     * admin_search method
     */
    public function admin_search()
    {
        
    }

    /**
     * admin_view method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_view($id = null)
    {
        if (!$this->Applicant->exists($id)) {
            throw new NotFoundException(__('Invalid applicant'));
        }
        $options = array('conditions' => array('Applicant.' . $this->Applicant->primaryKey => $id));
        $this->Applicant->recursive = 2;
        $this->set('applicant', $this->Applicant->find('first', $options));

        if ($this->request->is('ajax')) {
            $this->render('admin_view_ajax', 'ajax');
        }
    }

    /**
     * admin_add method
     *
     * @return void
     */
    public function admin_add()
    {

        if ($this->request->is('post')) {
            $this->Applicant->create();
            if ($this->Applicant->save($this->request->data)) {
                $this->Session->setFlash(__('The applicant has been saved.'), 'flash/success');
                return $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The applicant could not be saved. Please, try again.'), 'flash/error');
            }
        }

        // Get list of job with it's associated company name
        $this->loadModel('Job');
        $data = $this->Job->query("SELECT `Job`.`id`, CONCAT(`Job`.`title`, \" (\", `User`.`company_name`, \")\") as `display_name` FROM `jobs` as `Job`, `users` as `User` where `Job`.`user_id` = `User`.`id`");

        $job = array();
        if (!empty($data)) {
            foreach ($data as $key => $value) {
                $job[$data[$key]['Job']['id']] = $data[$key][0]['display_name'];
            }
        }

        $this->set('jobs', $job);


        if ($this->request->is('ajax')) {
            $this->render('admin_add_ajax', 'ajax');
        }
    }

    /**
     * admin_add_ajax method
     *
     * @return void
     */
    public function admin_add_ajax()
    {
        $this->Applicant->create();

        if ($this->Applicant->save($this->request->data)) {

            $this->Session->setFlash(__('The applicant has been saved.'), 'flash/success');

            $this->set('ok', true);
            $this->set('message', 'The applicant has been saved.');
            $this->set('data', $this->request->data);
        } else {
            $this->set('ok', false);
            $this->set('message', 'The applicant could not be saved. Please, try again.');
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    /**
     * admin_edit method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_edit($id = null)
    {

        if (!$this->Applicant->exists($id)) {
            throw new NotFoundException(__('Invalid applicant'));
        }
        if ($this->request->is(array('post', 'put'))) {
            if ($this->Applicant->save($this->request->data)) {
                $this->Session->setFlash(__('The applicant has been saved.'), 'flash/success');
                return $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The applicant could not be saved. Please, try again.'), 'flash/error');
            }
        } else {
            $options = array('conditions' => array('Applicant.' . $this->Applicant->primaryKey => $id));
            $this->request->data = $this->Applicant->find('first', $options);
            $this->request->data['Applicant']['password'] = '';
        }

        $this->set('jobs', $this->Applicant->Job->find('list'));


        if ($this->request->is('ajax')) {
            $this->render('admin_edit_ajax', 'ajax');
        }
    }

    /**
     * admin_edit_ajax method
     *
     * @return void
     */
    public function admin_edit_ajax($id = null)
    {
        if (!$this->Applicant->exists($id)) {
            $this->set('ok', false);
            $this->set('message', 'Invalid applicant');
        }

        if ($this->request->is(array('post', 'put'))) {

            if (empty($this->request->data['Applicant']['cv'])) {
                unset($this->request->data['Applicant']['cv']);
            }

            if ($this->Applicant->save($this->request->data)) {

                $this->Session->setFlash(__('The applicant has been saved.'), 'flash/success');

                $this->set('ok', true);
                $this->set('message', 'The applicant has been saved.');
                $this->set('data', $this->request->data);
            } else {
                $this->set('ok', false);
                $this->set('message', 'The applicant could not be saved. Please, try again.');
            }
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    /**
     * admin_delete method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_delete($id = null)
    {
        $this->Applicant->id = $id;
        if (!$this->Applicant->exists()) {
            throw new NotFoundException(__('Invalid applicant'));
        }
        $this->request->allowMethod('post', 'delete');
        if ($this->Applicant->delete()) {
            $this->Session->setFlash(__('The applicant has been deleted.'), 'flash/success');
        } else {
            $this->Session->setFlash(__('The applicant could not be deleted. Please, try again.'), 'flash/error');
        }

        $job_id = $this->request->query('job_id');
        if (!empty($job_id)) {
            return $this->redirect(array('controller' => 'jobs', 'action' => 'view', $job_id));
        }


        return $this->redirect(array('action' => 'index'));
    }

    /**
     * admin_bulk_delete method
     * 
     * @return type
     * @throws NotFoundException
     */
    public function admin_bulk_delete()
    {
        $this->request->allowMethod('post', 'delete');

        if (!isset($this->request->data['DeleteCheck'])) {
            throw new NotFoundException(__('Invalid record(s)'));
        }

        if (count($this->request->data['DeleteCheck']) == 0) {
            throw new NotFoundException(__('No record(s) selected'));
        }

        foreach ($this->request->data['DeleteCheck'] as $id) {
            $this->Applicant->delete($id);
        }

        $this->Session->setFlash(__('The applicant(s) has been deleted.'), 'flash/success');

        $job_id = $this->request->query('job_id');
        if (!empty($job_id)) {
            return $this->redirect(array('controller' => 'jobs', 'action' => 'view', $job_id));
        }

        return $this->redirect(array('action' => 'index'));
    }

    /**
     * add method
     *
     * @return void
     */
    public function add()
    {
        if ($this->request->is('post')) {
            $this->Applicant->create();
            if ($this->Applicant->save($this->request->data)) {
                $this->Session->setFlash(__('The applicant has been saved.'), 'flash/success');
                return $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The applicant could not be saved. Please, try again.'), 'flash/error');
            }
        }

        if ($this->request->is('ajax')) {
            $this->render('add_ajax', 'ajax');
        }
    }

    /**
     * add_ajax method
     *
     * @return void
     */
    public function add_ajax()
    {
        $this->Applicant->create();

        // set default status
        $this->request->data['Applicant']['status'] = 'New';

        if ($this->Applicant->save($this->request->data)) {

            $this->Session->setFlash(__('Your application has been submitted.'), 'flash_website/success');

            $this->set('ok', true);
            $this->set('message', 'Your application has been submitted.');
            $this->set('data', $this->request->data);

            // get application details
            $applicant = $this->Applicant->findById($this->Applicant->id);
            $this->loadModel('Job');
            $job = $this->Job->findById($applicant['Applicant']['job_id']);

            // notify applicant
            $email = new CakeEmail();
            $email->template('application_sent', 'default')
                ->config('default')
                ->helpers(array('Html'))
                ->emailFormat('html')
                ->subject(__('Application Sent') . ' - ' . $job['User']['company_name'])
                ->to($this->request->data['Applicant']['email'])
                ->from(Configure::read('Email.from_email'))
                ->viewVars(array('company_name' => $job['User']['company_name']))
                ->send();

            // notify user (company)
            $email->template('new_application_received', 'default')
                ->config('default')
                ->helpers(array('Html'))
                ->emailFormat('html')
                ->subject(__('New Application Received'))
                ->to($job['User']['email'])
                ->from(Configure::read('Email.from_email'))
                ->viewVars(array('applicant_name' => $this->request->data['Applicant']['name']))
                ->send();
            
        } else {
            $this->set('ok', false);
            $this->set('message', 'Your application could not be submitted. Please, try again.');
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    public function index($job_id)
    {
        $this->layout = 'website_user';

        if (!$this->Applicant->Job->exists($job_id)) {
            throw new NotFoundException(__('Invalid job'));
        }

        $this->Applicant->Job->recursive = 0;
        $this->set('job', $this->Applicant->Job->findById($job_id));

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->Applicant->parseCriteria($this->Prg->parsedParams());

        $this->Paginator->settings['conditions']['Applicant.job_id'] = $job_id;
        $this->Paginator->settings['conditions']['Job.user_id'] = $this->Auth->user('id');

        $this->Applicant->recursive = 0;
        $this->set('applicants', $this->Paginator->paginate());
    }

    /**
     * Delete method
     * 
     * @param type $id
     * @return type
     * @throws NotFoundException
     */
    public function delete($id = null)
    {
        $this->Applicant->id = $id;
        if (!$this->Applicant->exists()) {
            throw new NotFoundException(__('Invalid applicant'));
        }
        $this->request->allowMethod('post', 'delete');

        $job_id = $this->Applicant->field('job_id');

        if ($this->Applicant->delete()) {
            $this->Session->setFlash(__('The applicant has been deleted.'), 'flash_website/success');
        } else {
            $this->Session->setFlash(__('The applicant could not be deleted. Please, try again.'), 'flash_website/error');
        }

        return $this->redirect(array('action' => 'index', $job_id));
    }

    /**
     * edit method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function edit($id = null)
    {
        if (!$this->Applicant->exists($id)) {
            throw new NotFoundException(__('Invalid applicant'));
        }

        $options = array('conditions' => array('Applicant.' . $this->Applicant->primaryKey => $id));
        $this->request->data = $this->Applicant->find('first', $options);

        if ($this->request->is('ajax')) {
            $this->render('edit_ajax', 'ajax');
        }
    }

    /**
     * edit_ajax method
     *
     * @return void
     */
    public function edit_ajax()
    {
        if (!$this->Applicant->exists($this->request->data['Applicant']['id'])) {
            $this->set('ok', false);
            $this->set('message', 'Invalid applicant');
        }

        if ($this->request->is(array('post', 'put'))) {

            if (empty($this->request->data['Applicant']['cv'])) {
                unset($this->request->data['Applicant']['cv']);
            }

            if ($this->Applicant->save($this->request->data)) {

                $this->Session->setFlash(__('The applicant has been saved.'), 'flash_website/success');

                $this->set('ok', true);
                $this->set('message', 'The applicant has been saved.');
                $this->set('data', $this->request->data);
            } else {
                $this->set('ok', false);
                $this->set('message', 'The applicant could not be saved. Please, try again.');
            }
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    /**
     * view method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function view($id = null)
    {
        if (!$this->Applicant->exists($id)) {
            throw new NotFoundException(__('Invalid applicant'));
        }
        $options = array('conditions' => array('Applicant.' . $this->Applicant->primaryKey => $id));
        $this->Applicant->recursive = 2;
        $applicant = $this->Applicant->find('first', $options);
        $this->set('applicant', $applicant);
        $this->request->data = $applicant;

        if ($this->request->is('ajax')) {
            $this->render('view_ajax', 'ajax');
        }
    }
}
