<?php
App::uses('AppController', 'Controller');

/**
 * Jobs Controller
 *
 * Use DataTables
 * 
 * @property Job $Job
 * @property PaginatorComponent $Paginator
 */
class JobsController extends AppController
{

    /**
     * Components
     *
     * @var array
     */
    public $components = array('Paginator', 'Search.Prg', 'CsvView.CsvView', 'AjaxForms', 'RequestHandler' => array(
            'viewClassMap' => array('csv' => 'CsvView.Csv')
    ));

    public function beforeFilter()
    {
        parent::beforeFilter();

        $this->Auth->allow('view');

        // Only enable AjaxForms when needed
        $this->AjaxForms->enabled = false;

        if (isset($this->AjaxForms) && $this->RequestHandler->isAjax() && in_array($this->action, array('admin_add_ajax', 'admin_edit_ajax'))) {
            $this->AjaxForms->enabled = true;
        }

        // Use ajax layout when Ajax used
        if ($this->RequestHandler->isAjax()) {
            $this->layout = 'ajax';
        }
    }

    /**
     * Helpers
     */
    public $helpers = array('CustomForm');

    /**
     * Pagination Options
     */
    public $paginate = array(
        'limit' => 10,
        'order' => array(
            'Job.name' => 'asc'
        ),
        'paramType' => 'querystring',
    );

    public function admin_load_data()
    {
        ini_set('memory_limit', '-1');
        $this->Job->recursive = 0;

        $this->Prg->commonProcess();

        $jobs = $this->Job->find('all', array(
            'conditions' => $this->Job->parseCriteria($this->Prg->parsedParams()),
        ));

        $this->set('data', $jobs);
        $this->set('status', 'success');
        $this->set('message', 'Record(s) found');
        $this->set('_serialize', array('data', 'status', 'message'));
    }

    /**
     * index method
     *
     * @return void
     */
    public function admin_index_normaltable()
    {
        $this->Job->recursive = 0;

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->Job->parseCriteria($this->Prg->parsedParams());

        $this->set('jobs', $this->Paginator->paginate());
    }

    public function admin_index()
    {
        $this->Job->recursive = 0;

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->Job->parseCriteria($this->Prg->parsedParams());

        $this->set('jobs', $this->Paginator->paginate());
    }

    /**
     * admin_print method
     */
    public function admin_print()
    {
        Configure::write('debug', 0);
        $this->layout = 'print';
        $this->Job->recursive = 0;

        $this->pdfConfig = array(
            'orientation' => 'portrait',
            'filename' => 'Jobs_List',
        );

        $this->set('jobs', $this->Job->find('all', array('order' => 'Job.created DESC')));
    }

    /**
     * admin_export method
     */
    public function admin_export()
    {
        $results = $this->Job->find('all');

        $excludePaths = array('Job.password'); // Exclude all id fields

        $this->CsvView->quickExport($results, $excludePaths);
    }

    /**
     * admin_search method
     */
    public function admin_search()
    {
        $this->set('users', $this->Job->User->find('list'));

        if ($this->request->is('ajax')) {
            $this->render('admin_search_ajax', 'ajax');
        }
    }

    /**
     * admin_view method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_view($id = null)
    {
        if (!$this->Job->exists($id)) {
            throw new NotFoundException(__('Invalid job'));
        }
        $options = array('conditions' => array('Job.' . $this->Job->primaryKey => $id));
        $this->set('job', $this->Job->find('first', $options));

        if ($this->request->is('ajax')) {
            $this->render('admin_view_ajax', 'ajax');
        }
    }

    /**
     * admin_add method
     *
     * @return void
     */
    public function admin_add()
    {

        if ($this->request->is('post')) {
            $this->Job->create();
            if ($this->Job->save($this->request->data)) {
                $this->Session->setFlash(__('The job has been saved.'), 'flash/success');
                return $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The job could not be saved. Please, try again.'), 'flash/error');
            }
        }

        $this->set('users', $this->Job->User->find('list'));


        if ($this->request->is('ajax')) {
            $this->render('admin_add_ajax', 'ajax');
        }
    }

    /**
     * admin_add_ajax method
     *
     * @return void
     */
    public function admin_add_ajax()
    {
        $this->Job->create();

        if (!empty($this->request->data['Job']['is_remote'])) {
            $this->request->data['Job']['location'] = 'Anywhere';
            $this->request->data['Job']['country'] = 'Remote';
        }

        if ($this->Job->save($this->request->data)) {

            $this->Session->setFlash(__('The job has been saved.'), 'flash/success');

            $this->set('ok', true);
            $this->set('message', 'The job has been saved.');
            $this->set('data', $this->request->data);
        } else {
            $this->set('ok', false);
            $this->set('message', 'The job could not be saved. Please, try again.');
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    /**
     * admin_edit method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_edit($id = null)
    {

        if (!$this->Job->exists($id)) {
            throw new NotFoundException(__('Invalid job'));
        }
        if ($this->request->is(array('post', 'put'))) {
            if ($this->Job->save($this->request->data)) {
                $this->Session->setFlash(__('The job has been saved.'), 'flash/success');
                return $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The job could not be saved. Please, try again.'), 'flash/error');
            }
        } else {
            $options = array('conditions' => array('Job.' . $this->Job->primaryKey => $id));
            $this->request->data = $this->Job->find('first', $options);
            $this->request->data['Job']['password'] = '';
        }

        $this->set('users', $this->Job->User->find('list'));

        if ($this->request->is('ajax')) {
            $this->render('admin_edit_ajax', 'ajax');
        }
    }

    /**
     * admin_edit_ajax method
     *
     * @return void
     */
    public function admin_edit_ajax($id = null)
    {
        if (!$this->Job->exists($id)) {
            $this->set('ok', false);
            $this->set('message', 'Invalid job');
        }

        if ($this->request->is(array('post', 'put'))) {

            if (!empty($this->request->data['Job']['is_remote'])) {
                $this->request->data['Job']['location'] = 'Anywhere';
                $this->request->data['Job']['country'] = 'Remote';
            }

            if ($this->Job->save($this->request->data)) {

                $this->Session->setFlash(__('The job has been saved.'), 'flash/success');

                $this->set('ok', true);
                $this->set('message', 'The job has been saved.');
                $this->set('data', $this->request->data);
            } else {
                $this->set('ok', false);
                $this->set('message', 'The job could not be saved. Please, try again.');
            }
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    /**
     * admin_delete method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_delete($id = null)
    {
        $this->Job->id = $id;
        if (!$this->Job->exists()) {
            throw new NotFoundException(__('Invalid job'));
        }
        $this->request->allowMethod('post', 'delete');
        if ($this->Job->delete()) {
            $this->Session->setFlash(__('The job has been deleted.'), 'flash/success');
        } else {
            $this->Session->setFlash(__('The job could not be deleted. Please, try again.'), 'flash/error');
        }

        $user_id = $this->request->query('user_id');
        if (!empty($user_id)) {
            return $this->redirect(array('controller' => 'users', 'action' => 'view', $user_id));
        }


        return $this->redirect(array('action' => 'index'));
    }

    /**
     * admin_bulk_delete method
     * 
     * @return type
     * @throws NotFoundException
     */
    public function admin_bulk_delete()
    {
        $this->request->allowMethod('post', 'delete');

        if (!isset($this->request->data['DeleteCheck'])) {
            throw new NotFoundException(__('Invalid record(s)'));
        }

        if (count($this->request->data['DeleteCheck']) == 0) {
            throw new NotFoundException(__('No record(s) selected'));
        }

        foreach ($this->request->data['DeleteCheck'] as $id) {
            $this->Job->delete($id);
        }

        $this->Session->setFlash(__('The job(s) has been deleted.'), 'flash/success');


        $user_id = $this->request->query('user_id');
        if (!empty($user_id)) {
            return $this->redirect(array('controller' => 'users', 'action' => 'view', $user_id));
        }


        return $this->redirect(array('action' => 'index'));
    }

    /**
     * add method
     *
     * @return void
     */
    public function add()
    {
        $this->layout = 'website_user';

        $this->checkingEntry();

        if ($this->request->is('post')) {
            $this->Job->create();

            if (!empty($this->request->data['Job']['is_remote'])) {
                $this->request->data['Job']['location'] = 'Anywhere';
                $this->request->data['Job']['country'] = 'Remote';
            }

            $this->request->data['Job']['user_id'] = $this->Auth->user('id');

            if ($this->Job->save($this->request->data)) {
                $this->Session->setFlash(__('The job has been saved.'), 'flash_website/success');
                return $this->redirect(array('action' => 'view', $this->Job->field('slug')));
            } else {
                $this->Session->setFlash(__('The job could not be saved. Please, try again.'), 'flash_website/error');
            }
        }
    }

    public function checkingEntry()
    {
        $is_charge = true;

        // check for limited free listing
        if ($this->systemSettings['is_limited_free']) {
            // check current listing count
            $this->loadModel('Job');
            $count_jobs = $this->Job->find('count');

            if ($count_jobs < $this->systemSettings['free_items_count']) {
                $is_charge = false;
            }
        }

        if ($is_charge) {
            // check for fee
            if ($this->systemSettings['is_fee']) {
                // check user subscription
                $this->loadModel('User');
                $user = $this->User->findById($this->Auth->user('id'));

                if (!$user['User']['is_subscribed']) {
                    if ($user['User']['is_expired']) {
                        $this->redirect('/payments/pay');
                    }
                }
            }
        }
    }

    /**
     * edit method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function edit($id = null)
    {
        $this->layout = 'website_user';

        if (!$this->Job->exists($id)) {
            throw new NotFoundException(__('Invalid job'));
        }
        if ($this->request->is(array('post', 'put'))) {
            if ($this->Job->save($this->request->data)) {
                $this->Session->setFlash(__('The job has been saved.'), 'flash_website/success');
                return $this->redirect(array('action' => 'view', $this->Job->field('slug')));
            } else {
                $this->Session->setFlash(__('The job could not be saved. Please, try again.'), 'flash_website/error');
            }
        } else {
            $options = array('conditions' => array('Job.' . $this->Job->primaryKey => $id));
            $this->request->data = $this->Job->find('first', $options);
        }
    }

    /**
     * view method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function view($slug = null)
    {
        $this->layout = 'website_blank';

        $job = $this->Job->findBySlug(trim($slug));

        if (empty($job)) {
            throw new NotFoundException(__('Invalid job'));
        }
        $this->set('job', $job);
    }

    public function index()
    {
        $this->layout = 'website_user';

        $this->Job->recursive = 0;

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->Job->parseCriteria($this->Prg->parsedParams());

        $this->Paginator->settings['conditions']['user_id'] = $this->Auth->user('id');

        $this->set('jobs', $this->Paginator->paginate());
    }

    /**
     * bulk_delete method
     * 
     * @return type
     * @throws NotFoundException
     */
    public function bulk_delete()
    {
        $this->request->allowMethod('post', 'delete');

        if (!isset($this->request->data['DeleteCheck'])) {
            throw new NotFoundException(__('Invalid record(s)'));
        }

        if (count($this->request->data['DeleteCheck']) == 0) {
            throw new NotFoundException(__('No record(s) selected'));
        }

        foreach ($this->request->data['DeleteCheck'] as $id) {
            $this->Job->delete($id);
        }

        $this->Session->setFlash(__('The job(s) has been deleted.'), 'flash_website/success');
        return $this->redirect(array('action' => 'index'));
    }

    /**
     * delete method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function delete($id = null)
    {
        $this->Job->id = $id;
        if (!$this->Job->exists()) {
            throw new NotFoundException(__('Invalid job'));
        }
        $this->request->allowMethod('post', 'delete');
        if ($this->Job->delete()) {
            $this->Session->setFlash(__('The job has been deleted.'), 'flash_website/success');
        } else {
            $this->Session->setFlash(__('The job could not be deleted. Please, try again.'), 'flash_website/error');
        }

        return $this->redirect(array('action' => 'index'));
    }
}
