<?php
App::uses('AppController', 'Controller');

/**
 * Pages Controller
 *
 * Use DataTables
 * 
 * @property Page $Page
 * @property PaginatorComponent $Paginator
 */
class PagesController extends AppController
{

    /**
     * Components
     *
     * @var array
     */
    public $components = array('Paginator', 'Search.Prg', 'CsvView.CsvView', 'AjaxForms', 'RequestHandler' => array(
            'viewClassMap' => array('csv' => 'CsvView.Csv')
    ));

    public function beforeFilter()
    {
        parent::beforeFilter();

        $this->Auth->allow('home', 'unauthorized', 'view');

        // Only enable AjaxForms when needed
        $this->AjaxForms->enabled = false;

        if (isset($this->AjaxForms) && $this->RequestHandler->isAjax() && in_array($this->action, array('admin_add_ajax', 'admin_edit_ajax'))) {
            $this->AjaxForms->enabled = true;
        }

        // Use ajax layout when Ajax used
        if ($this->RequestHandler->isAjax()) {
            $this->layout = 'ajax';
        }
    }

    /**
     * Helpers
     */
    public $helpers = array('CustomForm');

    /**
     * Pagination Options
     */
    public $paginate = array(
        'limit' => 10,
        'order' => array(
            'Page.name' => 'asc'
        ),
        'paramType' => 'querystring',
    );

    public function admin_load_data()
    {
        ini_set('memory_limit', '-1');
        $this->Page->recursive = 0;

        $this->Prg->commonProcess();

        $pages = $this->Page->find('all', array(
            'conditions' => $this->Page->parseCriteria($this->Prg->parsedParams()),
        ));

        $this->set('data', $pages);
        $this->set('status', 'success');
        $this->set('message', 'Record(s) found');
        $this->set('_serialize', array('data', 'status', 'message'));
    }

    /**
     * index method
     *
     * @return void
     */
    public function admin_index_normaltable()
    {
        $this->Page->recursive = 0;

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->Page->parseCriteria($this->Prg->parsedParams());

        $this->set('pages', $this->Paginator->paginate());
    }

    public function admin_index()
    {
        $this->Page->recursive = 0;

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->Page->parseCriteria($this->Prg->parsedParams());

        $this->set('pages', $this->Paginator->paginate());
    }

    /**
     * admin_print method
     */
    public function admin_print()
    {
        Configure::write('debug', 0);
        $this->layout = 'print';
        $this->Page->recursive = 0;

        $this->pdfConfig = array(
            'orientation' => 'portrait',
            'filename' => 'Pages_List',
        );

        $this->set('pages', $this->Page->find('all', array('order' => 'Page.name ASC')));
    }

    /**
     * admin_export method
     */
    public function admin_export()
    {
        $results = $this->Page->find('all');

        $excludePaths = array('Page.password'); // Exclude all id fields

        $this->CsvView->quickExport($results, $excludePaths);
    }

    /**
     * admin_search method
     */
    public function admin_search()
    {
        
    }

    /**
     * admin_view method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_view($id = null)
    {
        if (!$this->Page->exists($id)) {
            throw new NotFoundException(__('Invalid page'));
        }
        $options = array('conditions' => array('Page.' . $this->Page->primaryKey => $id));
        $this->set('page', $this->Page->find('first', $options));

        if ($this->request->is('ajax')) {
            $this->render('admin_view_ajax', 'ajax');
        }
    }

    /**
     * admin_add method
     *
     * @return void
     */
    public function admin_add()
    {

        if ($this->request->is('post')) {
            $this->Page->create();
            if ($this->Page->save($this->request->data)) {
                $this->Session->setFlash(__('The page has been saved.'), 'flash/success');
                return $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The page could not be saved. Please, try again.'), 'flash/error');
            }
        }



        if ($this->request->is('ajax')) {
            $this->render('admin_add_ajax', 'ajax');
        }
    }

    /**
     * admin_add_ajax method
     *
     * @return void
     */
    public function admin_add_ajax()
    {
        $this->Page->create();

        if ($this->Page->save($this->request->data)) {

            $this->Session->setFlash(__('The page has been saved.'), 'flash/success');

            $this->set('ok', true);
            $this->set('message', 'The page has been saved.');
            $this->set('data', $this->request->data);
        } else {
            $this->set('ok', false);
            $this->set('message', 'The page could not be saved. Please, try again.');
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    /**
     * admin_edit method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_edit($id = null)
    {

        if (!$this->Page->exists($id)) {
            throw new NotFoundException(__('Invalid page'));
        }
        if ($this->request->is(array('post', 'put'))) {
            if ($this->Page->save($this->request->data)) {
                $this->Session->setFlash(__('The page has been saved.'), 'flash/success');
                return $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The page could not be saved. Please, try again.'), 'flash/error');
            }
        } else {
            $options = array('conditions' => array('Page.' . $this->Page->primaryKey => $id));
            $this->request->data = $this->Page->find('first', $options);
            $this->request->data['Page']['password'] = '';
        }

        if ($this->request->is('ajax')) {
            $this->render('admin_edit_ajax', 'ajax');
        }
    }

    /**
     * admin_edit_ajax method
     *
     * @return void
     */
    public function admin_edit_ajax($id = null)
    {
        if (!$this->Page->exists($id)) {
            $this->set('ok', false);
            $this->set('message', 'Invalid page');
        }

        if ($this->request->is(array('post', 'put'))) {
            if ($this->Page->save($this->request->data)) {

                $this->Session->setFlash(__('The page has been saved.'), 'flash/success');

                $this->set('ok', true);
                $this->set('message', 'The page has been saved.');
                $this->set('data', $this->request->data);
            } else {
                $this->set('ok', false);
                $this->set('message', 'The page could not be saved. Please, try again.');
            }
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    /**
     * admin_delete method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_delete($id = null)
    {
        $this->Page->id = $id;
        if (!$this->Page->exists()) {
            throw new NotFoundException(__('Invalid page'));
        }
        $this->request->allowMethod('post', 'delete');
        if ($this->Page->delete()) {
            $this->Session->setFlash(__('The page has been deleted.'), 'flash/success');
        } else {
            $this->Session->setFlash(__('The page could not be deleted. Please, try again.'), 'flash/error');
        }

        return $this->redirect(array('action' => 'index'));
    }

    /**
     * admin_bulk_delete method
     * 
     * @return type
     * @throws NotFoundException
     */
    public function admin_bulk_delete()
    {
        $this->request->allowMethod('post', 'delete');

        if (!isset($this->request->data['DeleteCheck'])) {
            throw new NotFoundException(__('Invalid record(s)'));
        }

        if (count($this->request->data['DeleteCheck']) == 0) {
            throw new NotFoundException(__('No record(s) selected'));
        }

        foreach ($this->request->data['DeleteCheck'] as $id) {
            $this->Page->delete($id);
        }

        $this->Session->setFlash(__('The page(s) has been deleted.'), 'flash/success');

        return $this->redirect(array('action' => 'index'));
    }

    /**
     * view method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function view($slug = null)
    {
        $this->layout = 'website';

        $page = $this->Page->findBySlug($slug);

        if (empty($page)) {
            throw new NotFoundException(__('Invalid page'));
        }

        $this->set('page', $page);
    }

    public function home()
    {
        $this->layout = 'website';

        $this->loadModel('Job');
        
        // Get featured jobs
        $featured_jobs = $this->Job->find('all', array(
            'conditions' => array(
                'Job.is_featured' => 1,
            ),
            'order' => 'Job.created DESC',
        ));
        
        $this->set('featured_jobs', $featured_jobs);

        // Get jobs
        $this->Prg->commonProcess();
        $this->Paginator->settings = array(
            'limit' => 10,
            'order' => array(
                'Job.created' => 'desc'
            ),
            'paramType' => 'querystring',
        );
        
        $this->Paginator->settings['conditions'] = $this->Job->parseCriteria($this->Prg->parsedParams());

        $this->Paginator->settings['conditions']['DATEDIFF(CURDATE(), DATE(Job.created)) <='] = $this->systemSettings['jobs_expiry_days'];
        $this->Paginator->settings['conditions']['Job.is_featured'] = 0;

        $this->set('jobs', $this->Paginator->paginate('Job'));

        
    }

    public function unauthorized()
    {
        $this->layout = 'website';
    }
}
