<?php
App::uses('AppController', 'Controller');

/**
 * Payments Controller
 *
 * Use DataTables
 * 
 * @property Payment $Payment
 * @property PaginatorComponent $Paginator
 */
class PaymentsController extends AppController
{

    /**
     * Components
     *
     * @var array
     */
    public $components = array('Paginator', 'Search.Prg', 'CsvView.CsvView', 'AjaxForms', 'RequestHandler' => array(
            'viewClassMap' => array('csv' => 'CsvView.Csv')
    ));

    /**
     * Helpers
     */
    public $helpers = array('CustomForm', 'Number', 'PaypalIpn.Paypal');

    public function beforeFilter()
    {
        parent::beforeFilter();

        // Only enable AjaxForms when needed
        $this->AjaxForms->enabled = false;

        if (isset($this->AjaxForms) && $this->RequestHandler->isAjax() && in_array($this->action, array('admin_add_ajax', 'admin_edit_ajax'))) {
            $this->AjaxForms->enabled = true;
        }

        // Use ajax layout when Ajax used
        if ($this->RequestHandler->isAjax()) {
            $this->layout = 'ajax';
        }
    }

    /**
     * Pagination Options
     */
    public $paginate = array(
        'limit' => 10,
        'order' => array(
            'Payment.name' => 'asc'
        ),
        'paramType' => 'querystring',
    );

    public function admin_load_data()
    {
        ini_set('memory_limit', '-1');
        $this->Payment->recursive = 0;

        $this->Prg->commonProcess();

        $payments = $this->Payment->find('all', array(
            'conditions' => $this->Payment->parseCriteria($this->Prg->parsedParams()),
        ));

        $this->set('data', $payments);
        $this->set('status', 'success');
        $this->set('message', 'Record(s) found');
        $this->set('_serialize', array('data', 'status', 'message'));
    }

    /**
     * index method
     *
     * @return void
     */
    public function admin_index_normaltable()
    {
        $this->Payment->recursive = 0;

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->Payment->parseCriteria($this->Prg->parsedParams());

        $this->set('payments', $this->Paginator->paginate());
    }

    public function admin_index()
    {
        $this->Payment->recursive = 0;

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->Payment->parseCriteria($this->Prg->parsedParams());

        $this->set('payments', $this->Paginator->paginate());
    }

    /**
     * admin_print method
     */
    public function admin_print()
    {
        Configure::write('debug', 0);
        $this->layout = 'print';
        $this->Payment->recursive = 0;

        $this->pdfConfig = array(
            'orientation' => 'portrait',
            'filename' => 'Payments_List',
        );

        $this->set('payments', $this->Payment->find('all', array('order' => 'Payment.payment_datetime DESC')));
    }

    /**
     * admin_export method
     */
    public function admin_export()
    {
        $results = $this->Payment->find('all');

        $excludePaths = array('Payment.password'); // Exclude all id fields

        $this->CsvView->quickExport($results, $excludePaths);
    }

    /**
     * admin_search method
     */
    public function admin_search()
    {
        $this->set('users', $this->Payment->User->find('list'));

        if ($this->request->is('ajax')) {
            $this->render('admin_search_ajax', 'ajax');
        }
    }

    /**
     * admin_view method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_view($id = null)
    {
        if (!$this->Payment->exists($id)) {
            throw new NotFoundException(__('Invalid payment'));
        }
        $options = array('conditions' => array('Payment.' . $this->Payment->primaryKey => $id));
        $this->set('payment', $this->Payment->find('first', $options));

        if ($this->request->is('ajax')) {
            $this->render('admin_view_ajax', 'ajax');
        }
    }

    /**
     * admin_add method
     *
     * @return void
     */
    public function admin_add()
    {

        if ($this->request->is('post')) {
            $this->Payment->create();
            if ($this->Payment->save($this->request->data)) {
                $this->Session->setFlash(__('The payment has been saved.'), 'flash/success');
                return $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The payment could not be saved. Please, try again.'), 'flash/error');
            }
        }

        $this->set('users', $this->Payment->User->find('list'));


        if ($this->request->is('ajax')) {
            $this->render('admin_add_ajax', 'ajax');
        }
    }

    /**
     * admin_add_ajax method
     *
     * @return void
     */
    public function admin_add_ajax()
    {
        $this->Payment->create();

        if ($this->Payment->save($this->request->data)) {

            $this->Session->setFlash(__('The payment has been saved.'), 'flash/success');

            $this->set('ok', true);
            $this->set('message', 'The payment has been saved.');
            $this->set('data', $this->request->data);
        } else {
            $this->set('ok', false);
            $this->set('message', 'The payment could not be saved. Please, try again.');
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    /**
     * admin_edit method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_edit($id = null)
    {

        if (!$this->Payment->exists($id)) {
            throw new NotFoundException(__('Invalid payment'));
        }
        if ($this->request->is(array('post', 'put'))) {
            if ($this->Payment->save($this->request->data)) {
                $this->Session->setFlash(__('The payment has been saved.'), 'flash/success');
                return $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The payment could not be saved. Please, try again.'), 'flash/error');
            }
        } else {
            $options = array('conditions' => array('Payment.' . $this->Payment->primaryKey => $id));
            $this->request->data = $this->Payment->find('first', $options);
            $this->request->data['Payment']['password'] = '';
        }

        $this->set('users', $this->Payment->User->find('list'));


        if ($this->request->is('ajax')) {
            $this->render('admin_edit_ajax', 'ajax');
        }
    }

    /**
     * admin_edit_ajax method
     *
     * @return void
     */
    public function admin_edit_ajax($id = null)
    {
        if (!$this->Payment->exists($id)) {
            $this->set('ok', false);
            $this->set('message', 'Invalid payment');
        }

        if ($this->request->is(array('post', 'put'))) {
            if ($this->Payment->save($this->request->data)) {

                $this->Session->setFlash(__('The payment has been saved.'), 'flash/success');

                $this->set('ok', true);
                $this->set('message', 'The payment has been saved.');
                $this->set('data', $this->request->data);
            } else {
                $this->set('ok', false);
                $this->set('message', 'The payment could not be saved. Please, try again.');
            }
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    /**
     * admin_delete method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_delete($id = null)
    {
        $this->Payment->id = $id;
        if (!$this->Payment->exists()) {
            throw new NotFoundException(__('Invalid payment'));
        }
        $this->request->allowMethod('post', 'delete');
        if ($this->Payment->delete()) {
            $this->Session->setFlash(__('The payment has been deleted.'), 'flash/success');
        } else {
            $this->Session->setFlash(__('The payment could not be deleted. Please, try again.'), 'flash/error');
        }

        $event_id = $this->request->query('event_id');
        if (!empty($event_id)) {
            return $this->redirect(array('controller' => 'events', 'action' => 'view', $event_id));
        }


        return $this->redirect(array('action' => 'index'));
    }

    /**
     * admin_bulk_delete method
     * 
     * @return type
     * @throws NotFoundException
     */
    public function admin_bulk_delete()
    {
        $this->request->allowMethod('post', 'delete');

        if (!isset($this->request->data['DeleteCheck'])) {
            throw new NotFoundException(__('Invalid record(s)'));
        }

        if (count($this->request->data['DeleteCheck']) == 0) {
            throw new NotFoundException(__('No record(s) selected'));
        }

        foreach ($this->request->data['DeleteCheck'] as $id) {
            $this->Payment->delete($id);
        }

        $this->Session->setFlash(__('The payment(s) has been deleted.'), 'flash/success');


        $event_id = $this->request->query('event_id');
        if (!empty($event_id)) {
            return $this->redirect(array('controller' => 'events', 'action' => 'view', $event_id));
        }


        return $this->redirect(array('action' => 'index'));
    }

    public function pay()
    {
        $this->layout = 'website';

        // create payment record
        $this->Payment->updatePayment(AuthComponent::user('id'), 'Pending');

        $payment = $this->Payment->find('first', array(
            'conditions' => array(
                'Payment.user_id' => AuthComponent::user('id'),
            ),
        ));

        $this->set('payment', $payment);
    }

    public function thankyou()
    {
        $this->layout = 'website';

        if ($this->Payment->isPaid(AuthComponent::user('id'))) {
            return $this->redirect(array('controller' => 'jobs', 'action' => 'add'));
        }
    }
}
