<?php
App::uses('AppController', 'Controller');
App::uses('CakeEmail', 'Network/Email');
App::uses('BlowfishPasswordHasher', 'Controller/Component/Auth');

/**
 * Users Controller
 *
 * @property User $User
 * @property PaginatorComponent $Paginator
 */
class UsersController extends AppController
{

    /**
     * Components
     *
     * @var array
     */
    public $components = array('Paginator', 'Search.Prg', 'CsvView.CsvView', 'AjaxForms', 'RequestHandler' => array(
            'viewClassMap' => array('csv' => 'CsvView.Csv')
    ));

    public function beforeFilter()
    {
        parent::beforeFilter();

        $this->Auth->allow('login', 'logout', 'register', 'change_password', 'forgot_password', 'forgot_password_step_2', 'add_default_user');

        // Only enable AjaxForms when needed
        $this->AjaxForms->enabled = false;

        if (isset($this->AjaxForms) && $this->RequestHandler->isAjax() && in_array($this->action, array('admin_add_ajax', 'admin_edit_ajax'))) {
            $this->AjaxForms->enabled = true;
        }

        // Use ajax layout when Ajax used
        if ($this->RequestHandler->isAjax()) {
            $this->layout = 'ajax';
        }
    }

    /**
     * Helpers
     */
    public $helpers = array('CustomForm');

    /**
     * Pagination Options
     */
    public $paginate = array(
        'limit' => 10,
        'order' => array(
            'User.name' => 'asc'
        ),
        'paramType' => 'querystring',
    );

    /**
     * admin_login method
     * 
     * @return void
     */
    public function admin_login()
    {
        $this->layout = 'blank';

        if ($this->request->is('post')) {
            if ($this->Auth->login()) {
                return $this->redirect($this->Auth->redirectUrl());
            }
            $this->Session->setFlash(
                __('Username or password is incorrect'), 'flash/error'
            );
        }
    }

    /**
     * admin_logout method
     * 
     * @return void
     */
    public function admin_logout()
    {
        return $this->redirect($this->Auth->logout());
    }

    /**
     * admin_register method
     */
    public function admin_register()
    {
        $this->layout = 'website';

        if ($this->request->is('post')) {
            $this->User->create();
            if ($this->User->save($this->request->data)) {
                $this->Session->setFlash(__('Your account has been created. You may login now.'), 'flash/success');
                return $this->redirect(array('action' => 'login'));
            } else {
                $this->Session->setFlash(__('The user could not be saved. Please, try again.'), 'flash/error');
            }
        }
    }

    public function admin_forgot_password()
    {
        $this->layout = 'blank';

        if ($this->request->is('post')) {
            $user = $this->User->findByEmail($this->request->data['User']['email']);

            if (empty($user)) {
                $this->Session->setFlash('This email does not exist in our database.', 'flash/error');
                $this->redirect(array('action' => 'forgot_password'));
            }

            $hash = $this->User->generateHashChangePassword();

            $data = array(
                'User' => array(
                    'id' => $user['User']['id'],
                    'hash_change_password' => $hash
                )
            );

            $this->User->save($data);

            $email = new CakeEmail();
            $email->template('forgot_password', 'default')
                ->config('default')
                ->helpers(array('Html'))
                ->emailFormat('html')
                ->subject(__('Reset password - ' . Configure::read('Application.name')))
                ->to($user['User']['email'])
                ->from(Configure::read('Email.from_email'))
                ->viewVars(array('hash' => $hash))
                ->send();

            $this->Session->setFlash('We have sent link to reset your password to your mailbox.', 'flash/success');
        }
    }

    /**
     * Step 2 to change the password.
     * This step verifies that the hash is valid, if it is, show the form to the user to inform your new password
     */
    public function admin_forgot_password_step_2($hash = null)
    {
        $this->layout = 'blank';

        $user = $this->User->findByHashChangePassword($hash);

        if ($user['User']['hash_change_password'] != $hash || empty($user)) {
            throw new NotFoundException(__('Link invalid'));
        }

        # Sends the hash to the form to check before changing the password
        $this->set('hash', $hash);

        $this->render('/Users/change_password');
    }

    public function admin_change_password()
    {
        $this->layout = 'blank';

        $user = $this->User->read(null, AuthComponent::user('id'));
        $this->set('user', $user);

        if ($this->request->is('post')) {
            if ($this->request->data['User']['password'] === $this->request->data['User']['re_password']) {
                if (AuthComponent::user('id')) {
                    $this->request->data['User']['id'] = AuthComponent::user('id');
                } else {
                    $user = $this->User->findByHashChangePassword($this->request->data['User']['hash']);

                    if (!empty($user)) {
                        $this->request->data['User']['id'] = $user['User']['id'];

                        $this->request->data['User']['hash_change_password'] = '';
                    } else {
                        throw new MethodNotAllowedException(__('Invalid action'));
                    }
                }

                if ($this->User->save($this->request->data)) {
                    $this->Session->setFlash('Password updated successfully!', 'flash/success');
                    return $this->redirect(array('action' => 'login'));
                }
            } else {
                $this->Session->setFlash('Passwords do not match.', 'flash/error');
            }
        }
    }

    public function admin_load_data()
    {
        ini_set('memory_limit', '-1');
        $this->User->recursive = 0;

        $this->Prg->commonProcess();

        $users = $this->User->find('all', array(
            'conditions' => $this->User->parseCriteria($this->Prg->parsedParams()),
        ));

        $this->set('data', $users);
        $this->set('status', 'success');
        $this->set('message', 'Record(s) found');
        $this->set('_serialize', array('data', 'status', 'message'));
    }

    /**
     * index method
     *
     * @return void
     */
    public function admin_index_normaltable()
    {
        $this->User->recursive = 0;

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->User->parseCriteria($this->Prg->parsedParams());

        $this->set('users', $this->Paginator->paginate());
    }

    public function admin_index()
    {
        $this->User->recursive = 0;

        $this->Prg->commonProcess();
        $this->Paginator->settings = $this->paginate;
        $this->Paginator->settings['conditions'] = $this->User->parseCriteria($this->Prg->parsedParams());

        $this->set('users', $this->Paginator->paginate());
    }

    /**
     * admin_print method
     */
    public function admin_print()
    {
        Configure::write('debug', 0);
        $this->layout = 'print';
        $this->User->recursive = 0;

        $this->pdfConfig = array(
            'orientation' => 'portrait',
            'filename' => 'Users_List',
        );

        $this->set('users', $this->User->find('all', array('order' => 'User.name ASC')));
    }

    /**
     * admin_export method
     */
    public function admin_export()
    {
        $results = $this->User->find('all');

        $excludePaths = array('User.password'); // Exclude all id fields

        $this->CsvView->quickExport($results, $excludePaths);
    }

    /**
     * admin_search method
     */
    public function admin_search()
    {
        if ($this->request->is('ajax')) {
            $this->render('admin_search_ajax', 'ajax');
        }
    }

    /**
     * admin_view method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_view($id = null)
    {
        if (!$this->User->exists($id)) {
            throw new NotFoundException(__('Invalid user'));
        }
        $options = array('conditions' => array('User.' . $this->User->primaryKey => $id));
        $this->set('user', $this->User->find('first', $options));

        if ($this->request->is('ajax')) {
            $this->render('admin_view_ajax', 'ajax');
        }
    }

    /**
     * admin_add method
     *
     * @return void
     */
    public function admin_add()
    {

        if ($this->request->is('post')) {
            $this->User->create();
            if ($this->User->save($this->request->data)) {
                $this->Session->setFlash(__('The user has been saved.'), 'flash/success');
                return $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The user could not be saved. Please, try again.'), 'flash/error');
            }
        }

        if ($this->request->is('ajax')) {
            $this->render('admin_add_ajax', 'ajax');
        }
    }

    /**
     * admin_add_ajax method
     *
     * @return void
     */
    public function admin_add_ajax()
    {
        $this->User->create();

        if (empty($this->request->data['User']['company_logo'])) {
            unset($this->request->data['User']['company_logo']);
        }

        if ($this->User->save($this->request->data)) {

            $this->Session->setFlash(__('The user has been saved.'), 'flash/success');

            $this->set('ok', true);
            $this->set('message', 'The user has been saved.');
            $this->set('data', $this->request->data);
        } else {
            $this->set('ok', false);
            $this->set('message', 'The user could not be saved. Please, try again.');
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    /**
     * admin_edit method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_edit($id = null)
    {

        if (!$this->User->exists($id)) {
            throw new NotFoundException(__('Invalid user'));
        }
        if ($this->request->is(array('post', 'put'))) {
            if ($this->User->save($this->request->data)) {
                $this->Session->setFlash(__('The user has been saved.'), 'flash/success');
                return $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The user could not be saved. Please, try again.'), 'flash/error');
            }
        } else {
            $options = array('conditions' => array('User.' . $this->User->primaryKey => $id));
            $this->request->data = $this->User->find('first', $options);
            $this->request->data['User']['password'] = '';
        }

        if ($this->request->is('ajax')) {
            $this->render('admin_edit_ajax', 'ajax');
        }
    }

    /**
     * admin_edit_ajax method
     *
     * @return void
     */
    public function admin_edit_ajax($id = null)
    {
        if (!$this->User->exists($id)) {
            $this->set('ok', false);
            $this->set('message', 'Invalid user');
        }

        if ($this->request->is(array('post', 'put'))) {

            if (empty($this->request->data['User']['company_logo'])) {
                unset($this->request->data['User']['company_logo']);
            }

            // check empty password
            if (empty($this->request->data['User']['password'])) {
                unset($this->request->data['User']['password']);
                unset($this->request->data['User']['password2']);
            }

            if ($this->User->save($this->request->data)) {

                $this->Session->setFlash(__('The user has been saved.'), 'flash/success');

                $this->set('ok', true);
                $this->set('message', 'The user has been saved.');
                $this->set('data', $this->request->data);
            } else {
                $this->set('ok', false);
                $this->set('message', 'The user could not be saved. Please, try again.');
            }
        }

        $this->set('_serialize', array('data', 'ok', 'message', 'invalid'));
    }

    /**
     * admin_delete method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function admin_delete($id = null)
    {
        $this->User->id = $id;
        if (!$this->User->exists()) {
            throw new NotFoundException(__('Invalid user'));
        }
        $this->request->allowMethod('post', 'delete');
        if ($this->User->delete()) {
            $this->Session->setFlash(__('The user has been deleted.'), 'flash/success');
        } else {
            $this->Session->setFlash(__('The user could not be deleted. Please, try again.'), 'flash/error');
        }
        return $this->redirect(array('action' => 'index'));
    }

    /**
     * admin_bulk_delete method
     * 
     * @return type
     * @throws NotFoundException
     */
    public function admin_bulk_delete()
    {
        $this->request->allowMethod('post', 'delete');

        if (!isset($this->request->data['DeleteCheck'])) {
            throw new NotFoundException(__('Invalid record(s)'));
        }

        if (count($this->request->data['DeleteCheck']) == 0) {
            throw new NotFoundException(__('No record(s) selected'));
        }

        foreach ($this->request->data['DeleteCheck'] as $id) {
            $this->User->delete($id);
        }

        $this->Session->setFlash(__('The user(s) has been deleted.'), 'flash/success');
        return $this->redirect(array('action' => 'index'));
    }

    /**
     * login method
     * 
     * @return void
     */
    public function login()
    {
        $this->layout = 'website';

        if ($this->request->is('post')) {
            if ($this->Auth->login()) {
                $_SESSION['KCFINDER'] = array(
                    'disabled' => false,
                );

                $this->Session->setFlash(__('You have successfully logged in.'), 'flash_website/success');

                return $this->redirect($this->Auth->redirectUrl());
            }
            $this->Session->setFlash(
                __('Username or password is incorrect'), 'flash_website/error'
            );
        }
    }

    /**
     * register method
     */
    public function register()
    {
        $this->layout = 'website';

        if ($this->request->is('post')) {

            $validRecaptcha = true;

            $whitelist = array(
                'localhost',
                '127.0.0.1',
                '::1'
            );

            // Disabled in localhost
            if (!in_array($_SERVER['REMOTE_ADDR'], $whitelist)) {
                $recaptcha = $this->data['g-recaptcha-response'];

                $google_url = "https://www.google.com/recaptcha/api/siteverify";
                $secret = $this->systemSettings['recaptcha_secretkey'];
                $ip = $_SERVER['REMOTE_ADDR'];
                $url = $google_url . "?secret=" . $secret . "&response=" . $recaptcha . "&remoteip=" . $ip;
                App::import('Vendor', 'curl');
                $res = getCurlData($url);
                $res = json_decode($res, true);

                if (empty($res['success'])) {
                    $this->Session->setFlash(__('The recaptcha is not valid. Please, try again.'), 'flash_website/error');
                    $validRecaptcha = false;
                }
            }

            if ($validRecaptcha) {
                $this->User->create();

                // check if no image selected
                if (empty($this->request->data['User']['company_logo'])) {
                    unset($this->request->data['User']['company_logo']);
                }

                if ($this->User->save($this->request->data)) {
                    $id = $this->User->id;

                    $this->request->data['User'] = array_merge(
                        $this->request->data['User'], array('id' => $id)
                    );

                    unset($this->request->data['User']['password']);

                    // notify user (company)
                    $email = new CakeEmail();
                    $email->template('new_registration', 'default')
                        ->config('default')
                        ->helpers(array('Html'))
                        ->emailFormat('html')
                        ->subject(__('New Registration'))
                        ->to($this->request->data['User']['email'])
                        ->from(Configure::read('Email.from_email'))
                        ->send();

                    // auto login
                    $this->Auth->login($this->request->data['User']);

                    $this->Session->setFlash(__('Your account has been created.'), 'flash_website/success');
                    return $this->redirect('/jobs/add');
                } else {
                    $this->Session->setFlash(__('The user could not be saved. Please, try again.'), 'flash_website/error');
                }
            }
        }
    }

    /**
     * profile method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function profile()
    {
        $this->layout = 'website_user';
        $id = $this->Auth->user('id');

        if (!$this->User->exists($id)) {
            throw new NotFoundException(__('Invalid user'));
        }
        if ($this->request->is(array('post', 'put'))) {
            // check empty password
            if (empty($this->request->data['User']['password'])) {
                unset($this->request->data['User']['password']);
                unset($this->request->data['User']['password2']);
            }

            if ($this->User->save($this->request->data)) {
                $this->request->data['User']['password'] = '';
                $this->request->data['User']['password2'] = '';
                $this->Session->setFlash(__('Your profile has been saved.'), 'flash_website/success');
            } else {
                $this->Session->setFlash(__('Your profile could not be saved. Please, try again.'), 'flash_website/error');
            }
        }

        $options = array('conditions' => array('User.' . $this->User->primaryKey => $id));
        $this->request->data = $this->User->find('first', $options);
        $this->request->data['User']['password'] = '';
    }

    /**
     * logout method
     * 
     * @return void
     */
    public function logout()
    {
        $this->Session->setFlash(__('You have successfully logout.'), 'flash_website/success');

        return $this->redirect($this->Auth->logout());
    }

    public function forgot_password()
    {
        $this->layout = 'website';

        if ($this->request->is('post')) {
            $user = $this->User->findByEmail($this->request->data['User']['email']);

            if (empty($user)) {
                $this->Session->setFlash('This email does not exist in our database.', 'flash/error');
                $this->redirect(array('action' => 'forgot_password'));
            }

            $hash = $this->User->generateHashChangePassword();

            $data = array(
                'User' => array(
                    'id' => $user['User']['id'],
                    'hash_change_password' => $hash
                )
            );

            $this->User->save($data);

            $email = new CakeEmail();
            $email->template('forgot_password', 'default')
                ->config('default')
                ->helpers(array('Html'))
                ->emailFormat('html')
                ->subject(__('Reset password - ' . Configure::read('Application.name')))
                ->to($user['User']['email'])
                ->from(Configure::read('Email.from_email'))
                ->viewVars(array('hash' => $hash))
                ->send();

            $this->Session->setFlash('We have sent link to reset your password to your mailbox.', 'flash_website/success');
        }
    }

    /**
     * Step 2 to change the password.
     * This step verifies that the hash is valid, if it is, show the form to the user to inform your new password
     */
    public function forgot_password_step_2($hash = null)
    {
        $this->layout = 'website';

        $user = $this->User->findByHashChangePassword($hash);

        if ($user['User']['hash_change_password'] != $hash || empty($user)) {
            throw new NotFoundException(__('Link invalid'));
        }

        # Sends the hash to the form to check before changing the password
        $this->set('hash', $hash);

        $this->render('/Users/change_password');
    }

    public function change_password()
    {
        $this->layout = 'website';

        $user = $this->User->read(null, AuthComponent::user('id'));
        $this->set('user', $user);

        if ($this->request->is('post')) {
            if ($this->request->data['User']['password'] === $this->request->data['User']['re_password']) {
                if (AuthComponent::user('id')) {
                    $this->request->data['User']['id'] = AuthComponent::user('id');
                } else {
                    $user = $this->User->findByHashChangePassword($this->request->data['User']['hash']);

                    if (!empty($user)) {
                        $this->request->data['User']['id'] = $user['User']['id'];

                        $this->request->data['User']['hash_change_password'] = '';
                    } else {
                        throw new MethodNotAllowedException(__('Invalid action'));
                    }
                }

                if ($this->User->save($this->request->data)) {
                    $this->Session->setFlash('Password updated successfully!', 'flash_website/success');
                    return $this->redirect(array('action' => 'login'));
                }
            } else {
                $this->Session->setFlash('Passwords do not match.', 'flash_website/error');
            }
        }
    }
}
