<?php
App::uses('AppModel', 'Model');
App::uses('BlowfishPasswordHasher', 'Controller/Component/Auth');

/**
 * Applicant Model
 *
 */
class Applicant extends AppModel
{

    /**
     * Display field
     *
     * @var string
     */
    public $displayField = 'name';
    public $actsAs = array(
        'Search.Searchable',
        'Upload.Upload' => array(
            'cv' => array(
                'fields' => array(
                    'dir' => 'cv_dir'
                ),
                'deleteOnUpdate' => true,
            )
        )
    );
    public $filterArgs = array(
//        'name' => array(
//            'type' => 'like',
//            'field' => 'name'
//        ),
        'id' => array(
            'type' => 'value',
            'field' => 'id'
        ),
        'job_id' => array(
            'type' => 'value',
            'field' => 'job_id'
        ),
        'name' => array(
            'type' => 'like',
            'field' => 'name'
        ),
        'email' => array(
            'type' => 'like',
            'field' => 'email'
        ),
        'phone' => array(
            'type' => 'like',
            'field' => 'phone'
        ),
        'cv' => array(
            'type' => 'like',
            'field' => 'cv'
        ),
        'cover_letter' => array(
            'type' => 'value',
            'field' => 'cover_letter'
        ),
        'created' => array(
            'type' => 'value',
            'field' => 'created'
        ),
        'modified' => array(
            'type' => 'value',
            'field' => 'modified'
        ),
        'term' => array(
            'type' => 'query',
            'method' => 'searchTermCondition'
        ),
    );

    public function searchTermCondition($data = array())
    {
        $filter = $data['term'];
        $conditions = array(
            'OR' => array(
                $this->alias . '.name LIKE' => '%' . $this->formatLike($filter) . '%',
//                $this->alias . '.username LIKE' => '%' . $this->formatLike($filter) . '%',
//                $this->alias . '.email LIKE' => '%' . $this->formatLike($filter) . '%',
            )
        );
        return $conditions;
    }

    /**
     * Validation rules
     *
     * @var array
     */
    public $validate = array(
        'name' => array(
            'notEmpty' => array(
                'rule' => array('notEmpty'),
                'message' => 'Your name is required',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
        'email' => array(
            'notEmpty' => array(
                'rule' => array('notEmpty'),
                'message' => 'Your email is required',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
            'email' => array(
                'rule' => array('email'),
                'message' => 'Your email is not valid',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
        'cover_letter' => array(
            'notEmpty' => array(
                'rule' => array('notEmpty'),
                'message' => 'Your cover letter is required',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
        'cv' => array(
            'isUnderPhpSizeLimit' => array(
                'rule' => 'isUnderPhpSizeLimit',
                'message' => 'File exceeds upload filesize limit',
                'on' => 'create',
            ),
            'isUnderFormSizeLimit' => array(
                'rule' => 'isUnderFormSizeLimit',
                'message' => 'File exceeds form upload filesize limit',
                'on' => 'create',
            ),
            'isCompletedUpload' => array(
                'rule' => 'isCompletedUpload',
                'message' => 'File was not successfully uploaded',
                'on' => 'create',
            ),
            'isValidExtension' => array(
                'rule' => array('isValidExtension', array('doc', 'docx', 'pdf'), false),
                'message' => 'File does not have a doc, docs, or pdf extension',
                'on' => 'create',
            ),
            'isBelowMaxSize' => array(
                'rule' => array('isBelowMaxSize', 5242880, false),
                'message' => 'File is larger than the maximum filesize (5mb)',
                'on' => 'create',
            ),
            'isAboveMinSize' => array(
                'rule' => array('isAboveMinSize', 1024, false),
                'message' => 'File is below the mimimum filesize',
                'on' => 'create',
            )
        )
    );

    /**
     * belongsTo associations
     *
     * @var array
     */
    public $belongsTo = array(
        'Job' => array(
            'className' => 'Job',
            'foreignKey' => 'job_id',
            'dependent' => false,
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'exclusive' => '',
            'finderQuery' => '',
            'counterQuery' => '',
            'counterCache' => true,
        ),
    );

    /**
     * hasMany associations
     *
     * @var array
     */
    public $hasMany = array(
    );

}
