<?php
App::uses('AppModel', 'Model');
App::uses('BlowfishPasswordHasher', 'Controller/Component/Auth');

/**
 * Job Model
 *
 */
class Job extends AppModel
{

    /**
     * Display field
     *
     * @var string
     */
    public $displayField = 'title';
//    public $virtualFields = array(
//        'display_name' => 'CONCAT(Job.title, " - ", User.company_name)'
//    );
    public $actsAs = array(
        'Search.Searchable'
    );
    public $filterArgs = array(
        'id' => array(
            'type' => 'value',
            'field' => 'id'
        ),
        'user_id' => array(
            'type' => 'value',
            'field' => 'user_id'
        ),
        'title' => array(
            'type' => 'like',
            'field' => 'title'
        ),
//        'slug' => array(
//            'type' => 'like',
//            'field' => 'slug'
//        ),
//        'description' => array(
//            'type' => 'value',
//            'field' => 'description'
//        ),
        'type' => array(
            'type' => 'like',
            'field' => 'type'
        ),
        'location' => array(
            'type' => 'like',
            'field' => 'location'
        ),
        'country' => array(
            'type' => 'like',
            'field' => 'country'
        ),
        'term' => array(
            'type' => 'query',
            'method' => 'searchTermCondition'
        ),
        'location_country' => array(
            'type' => 'query',
            'method' => 'searchLocationCountryCondition'
        ),
    );

    public function searchTermCondition($data = array())
    {
        $filter = $data['term'];
        $conditions = array(
            'OR' => array(
                $this->alias . '.title LIKE' => '%' . $this->formatLike($filter) . '%',
                $this->alias . '.description LIKE' => '%' . $this->formatLike($filter) . '%',
                $this->alias . '.description LIKE' => '%' . $this->formatLike($filter) . '%',
                $this->alias . '.location LIKE' => '%' . $this->formatLike($filter) . '%',
                $this->alias . '.country LIKE' => '%' . $this->formatLike($filter) . '%',
                'User.company_name LIKE' => '%' . $this->formatLike($filter) . '%',
            )
        );
        return $conditions;
    }

    public function searchLocationCountryCondition($data = array())
    {
        $filter = $data['location_country'];
        $conditions = array(
            'OR' => array(
                $this->alias . '.location LIKE' => '%' . $this->formatLike($filter) . '%',
                $this->alias . '.country LIKE' => '%' . $this->formatLike($filter) . '%',
            )
        );
        return $conditions;
    }

    /**
     * Validation rules
     *
     * @var array
     */
    public $validate = array(
        'user_id' => array(
            'notEmpty' => array(
                'rule' => array('notEmpty'),
            ),
        ),
        'title' => array(
            'notEmpty' => array(
                'rule' => array('notEmpty'),
                'message' => 'Title is required',
            ),
        ),
        'description' => array(
            'notEmpty' => array(
                'rule' => array('notEmpty'),
            ),
        ),
    );

    /**
     * belongsTo associations
     *
     * @var array
     */
    public $belongsTo = array(
        'User' => array(
            'className' => 'User',
            'foreignKey' => 'user_id',
            'dependent' => false,
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'exclusive' => '',
            'finderQuery' => '',
            'counterQuery' => ''
        ),
    );

    /**
     * hasMany associations
     *
     * @var array
     */
    public $hasMany = array(
        'Applicant' => array(
            'className' => 'Applicant',
            'foreignKey' => 'job_id',
            'conditions' => '',
            'order' => 'Applicant.created DESC',
            'limit' => '',
            'dependent' => false
        )
    );

    public function afterFind($results, $primary = false)
    {
        foreach ($results as $key => $val) {
            if (isset($val['Job']['created'])) {
                $start = date('Y-m-d', strtotime($val['Job']['created']));
                $end = date('Y-m-d');
                $start = new DateTime($start);
                $end = new DateTime($end);

                $days_posted = $start->diff($end)->days;

                $results[$key]['Job']['days_posted'] = $days_posted;

                if ($days_posted > 15) {
                    $results[$key]['Job']['is_expired'] = true;
                }
            }
        }
        return $results;
    }

    public function beforeSave($options = array())
    {
        // generate slug
        if (empty($this->data[$this->alias]['id'])) {
            if (isset($this->data[$this->alias]['title'])) {
                // check existing title
                $exist = false;
                $slug_ext = '';
                $no = 0;

                while (!$exist) {
                    $slug = Inflector::slug(strtolower($this->data[$this->alias]['title']), '-') . $slug_ext;
                    $job = $this->findBySlug($slug);

                    if (!empty($job)) {
                        $no += 1;
                        $slug_ext = '_' . $no;
                    } else {
                        $exist = true;
                    }
                }

                $this->data[$this->alias]['slug'] = $slug;
            }

            return true;
        }
    }
}
