<?php
App::uses('AppModel', 'Model');
App::uses('BlowfishPasswordHasher', 'Controller/Component/Auth');

/**
 * Payment Model
 *
 */
class Payment extends AppModel
{

    /**
     * Display field
     *
     * @var string
     */
    public $displayField = 'name';
    public $actsAs = array(
        'Search.Searchable'
    );
    public $filterArgs = array(
        'id' => array(
            'type' => 'like',
            'field' => 'id'
        ),
        'payment_datetime' => array(
            'type' => 'value',
            'field' => 'payment_datetime'
        ),
        'user_id' => array(
            'type' => 'value',
            'field' => 'user_id'
        ),
        'amount' => array(
            'type' => 'value',
            'field' => 'amount'
        ),
        'status' => array(
            'type' => 'like',
            'field' => 'status'
        ),
        'term' => array(
            'type' => 'query',
            'method' => 'searchTermCondition'
        ),
    );

    public function searchTermCondition($data = array())
    {
        $filter = $data['term'];
        $conditions = array(
            'OR' => array(
                $this->alias . '.name LIKE' => '%' . $this->formatLike($filter) . '%',
            )
        );
        return $conditions;
    }

    /**
     * Validation rules
     *
     * @var array
     */
    public $validate = array(
        'payment_datetime' => array(
            'notEmpty' => array(
                'rule' => array('notEmpty'),
                'message' => 'Date/time is required',
            ),
        ),
        'user_id' => array(
            'notEmpty' => array(
                'rule' => array('notEmpty'),
                'message' => 'User is required',
            ),
        ),
        'amount' => array(
            'notEmpty' => array(
                'rule' => array('notEmpty'),
                'message' => 'Amount is required',
            ),
        ),
        'status' => array(
            'notEmpty' => array(
                'rule' => array('notEmpty'),
                'message' => 'Status is required',
            ),
        ),
    );

    /**
     * belongsTo associations
     *
     * @var array
     */
    public $belongsTo = array(
        'User' => array(
            'className' => 'User',
            'foreignKey' => 'user_id',
            'dependent' => false,
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'exclusive' => '',
            'finderQuery' => '',
            'counterQuery' => ''
        ),
    );

    /**
     * hasMany associations
     *
     * @var array
     */
    public $hasMany = array(
    );

    public function afterFind($results, $primary = false)
    {
        foreach ($results as $key => $val) {
            if (isset($val['Payment']['payment_datetime'])) {
                $results[$key]['Payment']['payment_datetime'] = date("d-M-Y g:i A", strtotime($results[$key]['Payment']['payment_datetime']));
            }
        }
        return $results;
    }

    public function beforeSave($options = array())
    {
        if (!empty($this->data['Payment']['payment_datetime'])) {
            $this->data['Payment']['payment_datetime'] = $this->dateTimeFormatBeforeSave(
                $this->data['Payment']['payment_datetime']
            );
        }

        return true;
    }

    public function updatePayment($user_id, $status)
    {
        $SystemSetting = ClassRegistry::init('SystemSetting');
        $system_settings = $SystemSetting->findById(1);

        $payment = $this->find('first', array(
            'conditions' => array(
                'Payment.user_id' => $user_id,
                'datediff(now(), Payment.payment_datetime) < ' . $system_settings['SystemSetting']['fee_days']
            )
        ));

        if (!empty($payment)) {
            $this->id = $payment['Payment']['id'];
            $this->set('status', $status);
            $this->save();
        } else {
            $this->create();
            $this->set('payment_datetime', date("Y-m-d H:i:s"));
            $this->set('user_id', $user_id);

            $this->set('amount', $system_settings['SystemSetting']['fee_amount']);
            $this->set('status', $status);
            $this->save();
        }

        return true;
    }

    public function isPaid($user_id)
    {
        $payment = $this->find('first', array(
            'conditions' => array(
                'Payment.user_id' => $user_id,
                'Payment.status' => 'Paid',
            )
        ));

        if (!empty($payment)) {
            return true;
        }

        return false;
    }

    public function getMonthlyReport()
    {
        $end_date = date("Y-m-d H:i:s", strtotime("-1 year"));

        $payments = $this->find('all', array(
            'recursive' => -1,
            'fields' => 'DATE_FORMAT(Payment.payment_datetime, "%M") as month, YEAR(Payment.payment_datetime) as year, SUM(Payment.amount) as total',
            'conditions' => array(
                'Payment.status' => 'Paid',
                'Payment.payment_datetime >=' => $end_date,
            ),
            'order' => 'Payment.payment_datetime ASC',
            'group' => 'month, year',
        ));

        $temp = array();

        for ($i = 11; $i >= 0; $i--) {
            $temp[11 - $i]['year'] = date("Y", strtotime("-$i month"));
            $temp[11 - $i]['month'] = date("F", strtotime("-$i month"));
            $temp[11 - $i]['amount'] = 0.0;
        }

        if (count($payments) > 0) {
            foreach ($payments as $key1 => $value) {
                foreach ($temp as $key2 => $value) {
                    if (($payments[$key1][0]['month'] == $temp[$key2]['month']) && ($payments[$key1][0]['year'] == $temp[$key2]['year'])) {
                        $temp[$key2]['amount'] = $payments[$key1][0]['total'];
                    }
                }
            }
        }

        $report = array(
            'label' => '',
            'data' => '',
        );

        foreach ($temp as $key => $value) {
            if ($key > 0) {
                $report['label'] .= ',';
                $report['data'] .= ',';
            }

            $report['label'] .= '"' . $temp[$key]['month'] . '"';
            $report['data'] .= $temp[$key]['amount'];
        }

        return $report;
    }
}
