<?php
App::uses('AppModel', 'Model');
App::uses('BlowfishPasswordHasher', 'Controller/Component/Auth');

/**
 * SystemSetting Model
 *
 */
class SystemSetting extends AppModel
{

    /**
     * Display field
     *
     * @var string
     */
    public $displayField = 'name';
    public $actsAs = array(
        'Search.Searchable',
        'Upload.Upload' => array(
            'banner' => array(
                'fields' => array(
                    'dir' => 'banner_dir'
                ),
                'thumbnailSizes' => array(
                    'thumb' => '120x60'
                ),
                'deleteOnUpdate' => true,
                'thumbnailMethod' => 'php'
            )
        )
    );
    public $filterArgs = array(
        'id' => array(
            'type' => 'value',
            'field' => 'id'
        ),
    );

    public function searchTermCondition($data = array())
    {
        $filter = $data['term'];
        $conditions = array(
            'OR' => array(
                $this->alias . '.name LIKE' => '%' . $this->formatLike($filter) . '%',
            )
        );
        return $conditions;
    }

    /**
     * Validation rules
     *
     * @var array
     */
    public $validate = array(
        'banner' => array(
            'isUnderPhpSizeLimit' => array(
                'rule' => 'isUnderPhpSizeLimit',
                'message' => 'File exceeds upload filesize limit',
                'on' => 'create',
            ),
            'isUnderFormSizeLimit' => array(
                'rule' => 'isUnderFormSizeLimit',
                'message' => 'File exceeds form upload filesize limit',
                'on' => 'create',
            ),
            'isCompletedUpload' => array(
                'rule' => 'isCompletedUpload',
                'message' => 'File was not successfully uploaded',
                'on' => 'create',
            ),
            'isValidExtension' => array(
                'rule' => array('isValidExtension', array('jpg', 'gif', 'jpeg', 'png'), false),
                'message' => 'File does not have a jpg, gif, jpeg or png extension',
                'on' => 'create',
            ),
            'isBelowMaxSize' => array(
                'rule' => array('isBelowMaxSize', 5242880, false),
                'message' => 'File is larger than the maximum filesize (5mb)',
                'on' => 'create',
            ),
            'isAboveMinSize' => array(
                'rule' => array('isAboveMinSize', 1024, false),
                'message' => 'File is below the mimimum filesize',
                'on' => 'create',
            )
        )
    );

    /**
     * belongsTo associations
     *
     * @var array
     */
    public $belongsTo = array(
    );

    /**
     * hasMany associations
     *
     * @var array
     */
    public $hasMany = array(
    );

}
